/*******************************************************************************
 * Copyright (c) 2006-2010 Janis Schuller, nachtlicht-media
 * 
 * The software and the associated documentation are licensed under the 
 * GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007. For a detailed
 * license description, see LICENSE.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *******************************************************************************/
package de.nachtlicht_media.epoint.redcent;

import java.util.Date;

/**
 * The EpointRecord class represents ePoint transaction object that are published to the public records.
 * @author js
 *
 */
public class EpointRecord implements Comparable
{
	public static final int UNDEFINED = -1;
	public static final int EMIT = 1;
	public static final int EXCHANGE = 2;
	public static final int SPLIT = 3;
	public static final int MERGE = 4;
	public static final int INVALIDATION = 5;
	public static final int OBLIGATIONS_MERGE = 6;
	public static final int OBLIGATIONS_SPLIT = 7;
	
	private boolean isValid = false;
	private long serialNumber;
	private int issuingCause;
	private String id;
	private long value;
	private byte[] cert;
	private Date txDate;
	
	private String certificateType;
	private String templateReference;
	
	public EpointRecord(long serialNumber, int issuingCause, String id, long value, byte[] cert, Date txDate, boolean isValid, String type, String templateReference)
	{
		this.serialNumber = serialNumber;
		this.issuingCause = issuingCause;
		this.id = id;
		this.value = value;
		this.cert = cert;
		this.isValid = isValid;
		this.certificateType = type;
		this.templateReference = templateReference;
		this.txDate = txDate;
	}

	public byte[] getCert()
	{
		return cert;
	}
	public String getId()
	{
		return id;
	}
	public long getSerialNumber()
	{
		return serialNumber;
	}
	public long getValue()
	{
		return value;
	}
	
	public Date getTxDate() {
		return txDate;
	}
	
	public int compareTo(Object o)
	{
		if (o != null && o instanceof EpointRecord)
		{
			return getId().compareTo(((EpointRecord)o).getId());
		}
		
		return 0;
	}

	public void setCert(byte[] cert)
	{
		this.cert = cert;
	}

	public boolean isValid()
	{
		return isValid;
	}

	public void setValid(boolean isValid)
	{
		this.isValid = isValid;
	}

	public int getIssuingCause()
	{
		return issuingCause;
	}

	public String getTemplateReference()
	{
		return templateReference;
	}

	public String getCertificateType()
	{
		return certificateType;
	}
}
