/*******************************************************************************
 * Copyright (c) 2006-2010 Janis Schuller, nachtlicht-media
 * 
 * The software and the associated documentation are licensed under the 
 * GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007. For a detailed
 * license description, see LICENSE.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *******************************************************************************/
package de.nachtlicht_media.epoint.redcent.crypto;

import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.SignatureException;

import org.bouncycastle.openpgp.PGPException;
import org.bouncycastle.openpgp.PGPOnePassSignature;
import org.bouncycastle.openpgp.PGPPrivateKey;
import org.bouncycastle.openpgp.PGPSignature;
import org.bouncycastle.openpgp.PGPSignatureGenerator;
import org.bouncycastle.openpgp.PGPV3SignatureGenerator;

public class V3V4PGPSignatureGenerator
{
	public static final int V3 = 3;
	public static final int V4 = 4;
	
	private int signatureVersion;
	
	PGPSignatureGenerator v4Gen;
	PGPV3SignatureGenerator v3Gen;

	public V3V4PGPSignatureGenerator(
			int 	signatureVersion,
	        int     keyAlgorithm,
	        int     hashAlgorithm,
	        String  provider) throws NoSuchAlgorithmException, NoSuchProviderException, PGPException
	{
		this.signatureVersion = signatureVersion;
		if (signatureVersion != 3 && signatureVersion != 4)
			throw new IllegalArgumentException("Illegal signature version specified: " + this.signatureVersion);
		
		switch(signatureVersion)
		{
			case V3:
			{
				v3Gen = new PGPV3SignatureGenerator(keyAlgorithm, hashAlgorithm, provider);
				break;
			}
			case V4:
			{
				v4Gen = new PGPSignatureGenerator(keyAlgorithm, hashAlgorithm, provider);
				break;
			}
			default:
				throw new IllegalArgumentException("No signatureGenerator initialized.");
		}
	}
	
	public PGPSignature generate() throws PGPException, SignatureException
	{
		if (v3Gen != null)
			return v3Gen.generate();
		else if (v4Gen != null)
			return v4Gen.generate();
		else
			throw new IllegalArgumentException("No signatureGenerator initialized.");
	}

	public PGPOnePassSignature generateOnePassVersion(boolean isNested) throws PGPException
	{
		if (v3Gen != null)
			return v3Gen.generateOnePassVersion(isNested);
		else if (v4Gen != null)
			return v4Gen.generateOnePassVersion(isNested);
		else
			throw new IllegalArgumentException("No signatureGenerator initialized.");
	}

	public void initSign(int signatureType, PGPPrivateKey key) throws PGPException
	{
		if (v3Gen != null)
			v3Gen.initSign(signatureType, key);
		else if (v4Gen != null)
			v4Gen.initSign(signatureType, key);
		else
			throw new IllegalArgumentException("No signatureGenerator initialized.");
	}

	public void update(byte b) throws SignatureException
	{
		if (v3Gen != null)
			v3Gen.update(b);
		else if (v4Gen != null)
			v4Gen.update(b);
		else
			throw new IllegalArgumentException("No signatureGenerator initialized.");
	}
}
