/*******************************************************************************
 * Copyright (c) 2006-2010 Janis Schuller, nachtlicht-media
 * 
 * The software and the associated documentation are licensed under the 
 * GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007. For a detailed
 * license description, see LICENSE.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *******************************************************************************/
package de.nachtlicht_media.epoint.redcent.interfaces;

import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.security.NoSuchAlgorithmException;
import java.util.Collection;

import de.nachtlicht_media.epoint.redcent.EpointService;

public interface TemplateManager
{
	public void init(EpointService issuer) throws Exception;

	public String getTemplate(String certificateType);

	/**
	 * @return An unmodifiable Collection of the available certificate identifiers
	 */
	public Collection<String> getTemplateIdentifiers();

	public String getHashValueForCurrentVersion(String certificateType);
	
	/**
	 * 
	 * @param in The InputStream to read the template from. It is assumed that external templates 
	 * are encoded in UTF-8
	 * @return The template's internal representation
	 * @throws IOException
	 */
	public String readTemplate(InputStream in) throws IOException;
	
	/**
	 * Calculate a template's digest to compare templates
	 * @param template
	 * @return The template's digest
	 * @throws NoSuchAlgorithmException
	 * @throws UnsupportedEncodingException
	 */
	public String calcDigest(String template) throws NoSuchAlgorithmException, UnsupportedEncodingException;
}
