/*******************************************************************************
 * Copyright (c) 2006-2010 Janis Schuller, nachtlicht-media
 * 
 * The software and the associated documentation are licensed under the 
 * GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007. For a detailed
 * license description, see LICENSE.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *******************************************************************************/
package de.nachtlicht_media.epoint.redcent.messaging;

import java.text.NumberFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.Locale;
import java.util.TimeZone;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class StringDataMap extends LinkedHashMap
{
	private static final Log log = LogFactory.getLog(StringDataMap.class);
	public Object put(Object key, Object value)
	{
		if (!(key instanceof String))
			throw new IllegalArgumentException("Using objects as key entry in DataMaps is not permitted: " + key + " , " + value);
			
		if (value != null && !(value instanceof String))
		{
			if (value instanceof Date)
				value = formatDate((Date)value);
			else if (value instanceof Number)
				value = formatNumber((Number)value);

			if (!(value instanceof String))
			{
				value = String.valueOf(value);
				log.warn("Automatically converted value to String : " + value + " for key " + key);
			}
		}

		return put((String)key, (String)value);
	}
	
	public Object put(String key, String value)
	{
		return super.put(key, value);
	}
	
	public Date restoreDate(Object key) throws ParseException
	{
		SimpleDateFormat dateFormatter = new SimpleDateFormat(YYYY_MM_DD_HH_MM_SS_Z);
		dateFormatter.setTimeZone(TimeZone.getTimeZone("UTC"));

		if (!super.keySet().contains(key))
			return null;
		
		Object rawValue = super.get(key);
		
		if (String.valueOf(rawValue).trim().matches(DATE_PATTERN))
			return dateFormatter.parse(String.valueOf(rawValue).trim());
		
		throw new ParseException("The object referenced by key '"+key+"' does either not match the ePoint date format pattern or could not be parsed.",0);
	}
	
	public Number restoreNumber(Object key) throws ParseException
	{
		if (!super.keySet().contains(key))
			return null;
		
		Object rawValue = super.get(key);
		
		if (String.valueOf(rawValue).trim().matches(NUMBER_PATTERN))
			return nf.parse(String.valueOf(rawValue).trim());
		
		throw new ParseException("The object referenced by key '"+key+"' does either not match the ePoint number format pattern or could not be parsed.",0);
	}
	
	// The format for all dates in the ePoint ecosystem
	public static final String DATE_PATTERN = "^[0-9][0-9][0-9][0-9]-[0-1][0-9]-[0-3][0-9] [0-2][0-9]:[0-5][0-9]:[0-5][0-9] UTC$";
	private static final String YYYY_MM_DD_HH_MM_SS_Z = "yyyy-MM-dd HH:mm:ss z";

	public String formatDate(Date date)
	{
		SimpleDateFormat dateFormatter = new SimpleDateFormat(YYYY_MM_DD_HH_MM_SS_Z);
		dateFormatter.setTimeZone(TimeZone.getTimeZone("UTC"));
		if (date != null)
			return dateFormatter.format(date);
		
		return null;
	}
	
	public static final String NUMBER_PATTERN = "^-?[0-9]+(\\.[0-9]+)?$";
	private NumberFormat nf = NumberFormat.getNumberInstance(Locale.US);
	
	public String formatNumber(Number number)
	{
		nf.setGroupingUsed(false);
		return nf.format(number);
	}
}
