/*******************************************************************************
 * Copyright (c) 2006-2010 Janis Schuller, nachtlicht-media
 * 
 * The software and the associated documentation are licensed under the 
 * GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007. For a detailed
 * license description, see LICENSE.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *******************************************************************************/
package de.nachtlicht_media.epoint.redcent.servlet;

import javax.servlet.ServletContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.web.context.ServletContextAware;

import de.nachtlicht_media.epoint.redcent.interfaces.RedCentConfig;

public class RedCentServletConfig implements ServletContextAware, RedCentConfig
{
	private static final Log log = LogFactory.getLog(RedCentServletConfig.class);
	
	ServletContext context;
	
	public RedCentServletConfig()
	{
	}
	
	public void setServletContext(ServletContext context)
	{
		log.info("ServletContext " + context + " was injected.");
		this.context = context;
	}
	
	public String getMasterKeyId() {
		return context.getInitParameter("MasterKeyId");
	}
	
	/**
	 * @see de.nachtlicht_media.epoint.redcent.RedCent#getTemplateRealPath()
	 */
	public String getTemplateRealPath()
	{
		return context.getRealPath(context.getInitParameter("TemplatePath"));
	}

	/**
	 * @see de.nachtlicht_media.epoint.redcent.RedCent#getTrustedPublicKeyRingPath()
	 */
	public String getTrustedPublicKeyRingPath()
	{
		return context.getRealPath(context.getInitParameter("TrustedPublicKeyRingPath"));
	}

	/**
	 * @see de.nachtlicht_media.epoint.redcent.RedCent#getIssuersPublicKeyRingPath()
	 */
	public String getIssuersPublicKeyRingPath()
	{
		return context.getRealPath(context.getInitParameter("IssuersPublicKeyRingPath"));
	}

	/**
	 * @see de.nachtlicht_media.epoint.redcent.RedCent#getIssuersPrivateKeyRingPath()
	 */
	public String getIssuersPrivateKeyRingPath()
	{
		return context.getRealPath(context.getInitParameter("IssuersPrivateKeyRingPath"));
	}

	/**
	 * @see de.nachtlicht_media.epoint.redcent.RedCent#getIssuersKeyPassword()
	 */
	public String getIssuersPrivateKeyPassword()
	{
		return context.getInitParameter("IssuersPrivateKeyPassword");
	}

	/**
	 * @see de.nachtlicht_media.epoint.redcent.RedCent#getIssuerURL()
	 */
	public String getIssuerURL()
	{
		return context.getInitParameter("IssuerURL");
	}
	
	/**
	 * @see de.nachtlicht_media.epoint.redcent.RedCent#getIssuerName()
	 */
	public String getIssuerName()
	{
		return context.getInitParameter("IssuerName");
	}
	
	/**
	 * @see de.nachtlicht_media.epoint.redcent.RedCent#getSSLKey()
	 */
	public String getSSLKey()
	{
		return context.getInitParameter("SSLKey");
	}
	
	/**
	 * @see de.nachtlicht_media.epoint.redcent.RedCent#getIssuersEmailAddress()
	 */
	public String getIssuerEmail()
	{
		return context.getInitParameter("IssuerEmail");
	}

	public String getPublicKeyServerURL() {
		return context.getInitParameter("PublicKeyServerURL");
	}

	public Long getDefaultEmissionLimitLowerBound() {
		String val = context.getInitParameter("DefaultEmissionLimitLowerBound");
		
		if (val != null && val.trim().length() > 0)
			return new Long(Long.parseLong(val));
		else if (val == null) {
			log.warn("Parameter 'DefaultEmissionLimitLowerBound' is not set! Using 0 instead of null. To configure without lower bound, use an empty String!");
			return new Long(0);
		}
		
		return null;
	}

	public Long getDefaultEmissionLimitUpperBound() {
		String val = context.getInitParameter("DefaultEmissionLimitUpperBound");
		
		if (val != null && val.trim().length() > 0)
			return new Long(Long.parseLong(val));
		else if (val == null) {
			log.warn("Parameter 'DefaultEmissionLimitUpperBound' is not set! Using 0 instead of null. To configure without upper bound, use an empty String!");
			return new Long(0);
		}
		
		return null;
	}

	/**
	 * @see de.nachtlicht_media.epoint.redcent.interfaces.RedCentConfig#getNumberOfNotarySignaturesRequiredForDefaultAIRegistration()
	 */
	public Long getNumberOfNotarySignaturesRequiredForDefaultAIRegistration() {

		String val = context.getInitParameter("NumberOfNotarySignaturesRequiredForDefaultAIRegistration");
		
		if (val != null && val.trim().length() > 0) {
			Long parsedVal = new Long(Long.parseLong(val));
			
			if (parsedVal != null && parsedVal.longValue() <= 0) {
				log.warn("Value of Parameter 'NumberOfNotarySignaturesRequiredForDefaultAIRegistration' has invalid value " + parsedVal + ". It will be disabled.");
				parsedVal = null;
			}
			
			return parsedVal;
		}

		log.warn("Parameter 'NumberOfNotarySignaturesRequiredForDefaultAIRegistration' is not set! Registering authorized issuers by uploading public keys signed by notatiries is disabled.");
		return null;
	}
}
