/*******************************************************************************
 * Copyright (c) 2006-2010 Janis Schuller, nachtlicht-media
 * 
 * The software and the associated documentation are licensed under the 
 * GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007. For a detailed
 * license description, see LICENSE.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *******************************************************************************/
package de.nachtlicht_media.epoint.redcent.servlet.exception;

import java.util.LinkedHashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import de.nachtlicht_media.epoint.redcent.exception.BadsignatureException;
import de.nachtlicht_media.epoint.redcent.exception.DatabaseErrorException;
import de.nachtlicht_media.epoint.redcent.exception.ForbiddenExistingException;
import de.nachtlicht_media.epoint.redcent.exception.ForbiddenSelfException;
import de.nachtlicht_media.epoint.redcent.exception.InvalidTransactionError;
import de.nachtlicht_media.epoint.redcent.exception.MalformedRequestException;
import de.nachtlicht_media.epoint.redcent.exception.NotFoundException;

public class DefaultProcessingExceptionMapper implements ProcessingExceptionMapper
{
	private static final Log log = LogFactory.getLog(DefaultProcessingExceptionMapper.class);
	
	static class ExceptionMapping
	{
		Class fromClass;
		int status;
		String shortMessage;
		String message;
		public ExceptionMapping(Class fromClass, int status, String shortMessage, String message)
		{
			this.fromClass = fromClass;
			this.status = status;
			this.shortMessage = shortMessage;
			this.message = message;
		}
	}
	
	public ProcessingException getMappedException(Exception ex)
	{
		if (ex != null)
		{
			if (ex instanceof ProcessingException)
			{
				log.warn("invalid call to getMappedException(Exception ex)? -- Argument is a ProcessingException");
				return (ProcessingException)ex;
			}
			else if (hasMapping(ex))
			{
				ExceptionMapping mapping = getMapping(ex);
				return createMappedException(mapping.status, mapping.shortMessage, mapping.message, ex);
			}
			else
			{
				log.warn("No mapping found for exception " + ex, ex);
				return createMappedException(500, "Unmapped Exception", ex);
			}
		}
		else
			return new ProcessingException("Attempted to map illegal exception <null>.");
	}

	private ProcessingException createMappedException(int status, String shortMsg, Exception ex)
	{
		return createMappedException(status, shortMsg, null, ex);
	}

	private ProcessingException createMappedException(int status, String shortMsg, String message, Exception ex)
	{
		ProcessingException pEx =  new ProcessingException(status, shortMsg, message != null ? message : ex.getMessage());
		
		pEx.setStackTrace(ex.getStackTrace());
		pEx.initCause(ex);
		
		return pEx;
	}

	/*
	 *	Methods below could be replaced to allow external definition of exception mapping 
	 */
	
	// quick check whether there is a mapping for the requested type of exception
	
	private static final LinkedHashMap exMap = new LinkedHashMap();
	static
	{
		exMap.put(IllegalArgumentException.class, new ExceptionMapping(IllegalArgumentException.class, 500, "Illegal argument provided", null));
		exMap.put(DatabaseErrorException.class, new ExceptionMapping(DatabaseErrorException.class, 500, "Internal database error", null));
		exMap.put(NotFoundException.class, new ExceptionMapping(NotFoundException.class, 404, "Not found", null));
		exMap.put(ForbiddenExistingException.class, new ExceptionMapping(ForbiddenExistingException.class, 403, "Forbidden", null));
		exMap.put(ForbiddenSelfException.class, new ExceptionMapping(ForbiddenSelfException.class, 403, "Forbidden", null));
		exMap.put(MalformedRequestException.class, new ExceptionMapping(MalformedRequestException.class, 400, "Malformed request", null));
		exMap.put(BadsignatureException.class, new ExceptionMapping(BadsignatureException.class, 403, "Forbidden", null));
		exMap.put(InvalidTransactionError.class, new ExceptionMapping(InvalidTransactionError.class, 403, "Forbidden", null));
		
	}
	
	protected boolean hasMapping(Exception ex)
	{
		return exMap.containsKey(ex.getClass());
	}

	protected ExceptionMapping getMapping(Exception ex)
	{
		return (ExceptionMapping)exMap.get(ex.getClass());
	}
}
