/*******************************************************************************
 * Copyright (c) 2006-2010 Janis Schuller, nachtlicht-media
 * 
 * The software and the associated documentation are licensed under the 
 * GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007. For a detailed
 * license description, see LICENSE.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *******************************************************************************/
package de.nachtlicht_media.epoint.redcent.servlet.parameters;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import de.nachtlicht_media.epoint.redcent.interfaces.TransactionType;

public final class ServletInterfaceTransactions {
	
	public static class ServletInterfaceParameter {
		private String name;
		private String description;
		public ServletInterfaceParameter(String name, String description) {
			this.name = name;
			this.description = description;
		}
		
		/**
		 * @return the name
		 */
		public String getName() {
			return name;
		}
		
		/**
		 * @return the name
		 */
		public String key() {
			return name;
		}

		/**
		 * @param name the name to set
		 */
		public void setName(String name) {
			this.name = name;
		}

		/**
		 * @return the description
		 */
		public String getDescription() {
			return description;
		}

		/**
		 * @param description the description to set
		 */
		public void setDescription(String description) {
			this.description = description;
		}

		@Override
		public String toString() {
			return "[" + name + "] " + description;
		}
	}
	
	public static class InterfaceDescriptor {
		private final TransactionType transactionType;
		private final String relativeURL;
		private final String description;
		private final SignatureRequirement signatureRequirement;
		private final SecurityRequirement securityRequirement;
		private final List<ServletInterfaceParameter> parameters;
		private final List<ServletInterfaceParameter> optionalParameters;
		
		public InterfaceDescriptor(TransactionType transactionType, String relativeURL, String description, SignatureRequirement signatureRequirement, SecurityRequirement securityRequirement, List<ServletInterfaceParameter> parameters, List<ServletInterfaceParameter> optionalParameters) {
			this.transactionType = transactionType;
			this.relativeURL = relativeURL;
			this.description = description;
			this.signatureRequirement = signatureRequirement;
			this.securityRequirement = securityRequirement;
			this.parameters = parameters;
			this.optionalParameters = optionalParameters;
		}

		public InterfaceDescriptor(TransactionType transactionType, String relativeURL, String description, SignatureRequirement signatureRequirement, SecurityRequirement securityRequirement, List<ServletInterfaceParameter> parameters) {
			this(transactionType, relativeURL, description, signatureRequirement, securityRequirement, parameters, createParameterList());
		}
		
		/**
		 * @return the relativeURL
		 */
		public String getRelativeURL() {
			return relativeURL;
		}

		/**
		 * @return the relativeURL
		 */
		public String url() {
			return relativeURL;
		}
		
		/**
		 * @return the complete URL including parameter URL
		 */
		public String url(String url) {
			return url + (!url.endsWith("/") && !relativeURL.startsWith("/") ? "/" : "") + relativeURL;
		}
		
		/**
		 * @return the signatureRequirement
		 */
		public SignatureRequirement getSignatureRequirement() {
			return signatureRequirement;
		}

		/**
		 * @return the description
		 */
		public String getDescription() {
			return description;
		}

		/**
		 * @return the securityRequirement
		 */
		public SecurityRequirement getSecurityRequirement() {
			return securityRequirement;
		}

		/**
		 * @return the securityRequirement
		 */
		public boolean needsEncryption() {
			return SecurityRequirement.ENCRYPTED.equals(securityRequirement);
		}

		/**
		 * @return the parameters
		 */
		public List<ServletInterfaceParameter> getParameters() {
			return parameters;
		}

		/**
		 * @return the optionalParameters
		 */
		public List<ServletInterfaceParameter> getOptionalParameters() {
			return optionalParameters;
		}

		/**
		 * @return the transactionType
		 */
		public TransactionType getTransactionType() {
			return transactionType;
		}

		public static List<ServletInterfaceParameter> createParameterList() {List<ServletInterfaceParameter> list = new ArrayList<ServletInterfaceParameter>();return list;}
		public static List<ServletInterfaceParameter> createParameterList(ServletInterfaceParameter a) {List<ServletInterfaceParameter> list = new ArrayList<ServletInterfaceParameter>();list.add(a);return list;}
		public static List<ServletInterfaceParameter> createParameterList(ServletInterfaceParameter a, ServletInterfaceParameter b) {List<ServletInterfaceParameter> list = new ArrayList<ServletInterfaceParameter>();list.add(a);list.add(b);return list;}
		public static List<ServletInterfaceParameter> createParameterList(ServletInterfaceParameter a, ServletInterfaceParameter b, ServletInterfaceParameter c) {List<ServletInterfaceParameter> list = new ArrayList<ServletInterfaceParameter>();list.add(a);list.add(b);list.add(c);return list;}
		public static List<ServletInterfaceParameter> createParameterList(ServletInterfaceParameter a, ServletInterfaceParameter b, ServletInterfaceParameter c, ServletInterfaceParameter d) {List<ServletInterfaceParameter> list = new ArrayList<ServletInterfaceParameter>();list.add(a);list.add(b);list.add(c);list.add(d);return list;}
		public static List<ServletInterfaceParameter> createParameterList(ServletInterfaceParameter a, ServletInterfaceParameter b, ServletInterfaceParameter c, ServletInterfaceParameter d, ServletInterfaceParameter e) {List<ServletInterfaceParameter> list = new ArrayList<ServletInterfaceParameter>();list.add(a);list.add(b);list.add(c);list.add(d);list.add(e);return list;}
	}
	
	public static enum SignatureRequirement {
		NONE,
		AUTHORIZED_ISSUER,
		ISSUER_MASTER
	}

	public static enum SecurityRequirement {
		NONE,
		ENCRYPTED
	}

	
	public static final EmitInterfaceDescriptor EMIT = new EmitInterfaceDescriptor();
	public static final class EmitInterfaceDescriptor extends InterfaceDescriptor {
		public static final ServletInterfaceParameter DIGEST = new ServletInterfaceParameter("D", "digest");
		public static final ServletInterfaceParameter AMOUNT = new ServletInterfaceParameter("F", "amount");
		public static final ServletInterfaceParameter ISSUER = new ServletInterfaceParameter("K", "issuer");

		public EmitInterfaceDescriptor() {
			super(TransactionType.EMIT, "/emit", "emit ePoints", SignatureRequirement.AUTHORIZED_ISSUER, SecurityRequirement.ENCRYPTED ,createParameterList(DIGEST,AMOUNT,ISSUER));
		}
	}

	public static final GetTemplateInterfaceDescriptor GET_TEMPLATE = new GetTemplateInterfaceDescriptor();
	public static final class GetTemplateInterfaceDescriptor extends InterfaceDescriptor {
		public static final ServletInterfaceParameter ID = new ServletInterfaceParameter("ID", "template id");

		public GetTemplateInterfaceDescriptor() {
			super(TransactionType.GET_TEMPLATE, "/template", "get a template from the issuer", SignatureRequirement.NONE, SecurityRequirement.NONE, createParameterList(ID));
		}
	}

	public static final SecureMessageInterfaceDescriptor SECURE_MESSAGE = new SecureMessageInterfaceDescriptor();
	public static final class SecureMessageInterfaceDescriptor extends InterfaceDescriptor {
		public static final ServletInterfaceParameter ID = new ServletInterfaceParameter("ID", "template id");

		public SecureMessageInterfaceDescriptor() {
			super(TransactionType.SECURE_MESSAGE, "/secureMessage", "send pgp encrypted or signed message to the issuer", SignatureRequirement.NONE, SecurityRequirement.NONE, createParameterList(ID));
		}
	}

	public static final GetPublicKeyInterfaceDescriptor GET_PUBLIC_KEY = new GetPublicKeyInterfaceDescriptor();
	public static final class GetPublicKeyInterfaceDescriptor extends InterfaceDescriptor {
		public GetPublicKeyInterfaceDescriptor() {
			super(TransactionType.GET_PUBLIC_KEY, "/pubkey.asc", "get the issuer's public key", SignatureRequirement.NONE, SecurityRequirement.NONE, createParameterList());
		}
	}

	public static final GetInterfacesInterfaceDescriptor GET_INTERFACES = new GetInterfacesInterfaceDescriptor();
	public static final class GetInterfacesInterfaceDescriptor extends InterfaceDescriptor {
		public GetInterfacesInterfaceDescriptor() {
			super(TransactionType.GET_INTERFACES, "/interfaces", "get the issuer's public interfaces", SignatureRequirement.NONE, SecurityRequirement.NONE, createParameterList());
		}
	}

	public static final GetBalanceInterfaceDescriptor GET_BALANCE = new GetBalanceInterfaceDescriptor();
	public static final class GetBalanceInterfaceDescriptor extends InterfaceDescriptor {
		public GetBalanceInterfaceDescriptor() {
			super(TransactionType.GET_BALANCE, "/balance", "get a balance of emitted ePoints", SignatureRequirement.NONE, SecurityRequirement.NONE, createParameterList());
		}
	}

	public static final InfoInterfaceDescriptor INFO = new InfoInterfaceDescriptor();
	public static final class InfoInterfaceDescriptor extends InterfaceDescriptor {
		public static final ServletInterfaceParameter ID = new ServletInterfaceParameter("ID", "valid digest");
		public static final ServletInterfaceParameter KEY = new ServletInterfaceParameter("KEY", "obligation key");
		public static final ServletInterfaceParameter TYPE = new ServletInterfaceParameter("TYPE", "info type");
		public static final ServletInterfaceParameter CERT = new ServletInterfaceParameter("CERT", "certificate by serialnumber");
		public InfoInterfaceDescriptor() {
			super(TransactionType.INFO, "/info", "perform various info requests", SignatureRequirement.NONE, SecurityRequirement.ENCRYPTED, createParameterList(), createParameterList(CERT,ID,KEY,TYPE));
		}
	}
	
	public static final ExchangeInterfaceDescriptor EXCHANGE = new ExchangeInterfaceDescriptor();
	public static final class ExchangeInterfaceDescriptor extends InterfaceDescriptor {
		public static final ServletInterfaceParameter RAND = new ServletInterfaceParameter("B", "valid rand");
		public static final ServletInterfaceParameter DIGEST = new ServletInterfaceParameter("D", "new digest");
		public static final ServletInterfaceParameter REDIRECT = new ServletInterfaceParameter("G", "redirect");
		public ExchangeInterfaceDescriptor() {
			super(TransactionType.EXCHANGE, "/action", "exchange a rand for a new digest", SignatureRequirement.NONE, SecurityRequirement.ENCRYPTED, createParameterList(RAND,DIGEST), createParameterList(REDIRECT));
		}
	}
	
	public static final InvalidateInterfaceDescriptor INVALIDATE = new InvalidateInterfaceDescriptor();
	public static final class InvalidateInterfaceDescriptor extends InterfaceDescriptor {
		public static final ServletInterfaceParameter RAND = new ServletInterfaceParameter("B", "valid rand");
		public static final ServletInterfaceParameter AUTHORIZED_ISSUER = new ServletInterfaceParameter("A", "authorized issuer");
		public static final ServletInterfaceParameter ISSUER = new ServletInterfaceParameter("K", "issuer");
		public InvalidateInterfaceDescriptor() {
			super(TransactionType.INVALIDATE, "/invalidate", "invalidate ePoints", SignatureRequirement.NONE, SecurityRequirement.ENCRYPTED, createParameterList(RAND,AUTHORIZED_ISSUER,ISSUER));
		}
	}
	
	public static final SplitInterfaceDescriptor SPLIT = new SplitInterfaceDescriptor();
	public static final class SplitInterfaceDescriptor extends InterfaceDescriptor {
		public static final ServletInterfaceParameter RAND = new ServletInterfaceParameter("B", "valid rand");
		public static final ServletInterfaceParameter DIGEST = new ServletInterfaceParameter("D", "new digest");
		public static final ServletInterfaceParameter AMOUNT = new ServletInterfaceParameter("F", "amount for digest");
		public static final ServletInterfaceParameter REST_DIGEST = new ServletInterfaceParameter("C", "digest for remaining value");
		public SplitInterfaceDescriptor() {
			super(TransactionType.SPLIT, "/action", "split an existing ePoint certificate into two", SignatureRequirement.NONE, SecurityRequirement.ENCRYPTED, createParameterList(RAND,DIGEST,AMOUNT,REST_DIGEST));
		}
	}
	
	public static final ObligationsMergeInterfaceDescriptor OBLIGATIONS_MERGE = new ObligationsMergeInterfaceDescriptor();
	public static final class ObligationsMergeInterfaceDescriptor extends InterfaceDescriptor {
		public static final ServletInterfaceParameter RAND = new ServletInterfaceParameter("B", "valid rand");
		public static final ServletInterfaceParameter KEY = new ServletInterfaceParameter("KEY", "obligation key");
		public static final ServletInterfaceParameter REDIRECT = new ServletInterfaceParameter("G", "redirect");
		public ObligationsMergeInterfaceDescriptor() {
			super(TransactionType.OBLIGATIONS_MERGE, "/action", "merge a rand into an obligation", SignatureRequirement.NONE, SecurityRequirement.ENCRYPTED, createParameterList(RAND,KEY), createParameterList(REDIRECT));
		}
	}
	
	public static final UpdateAIInterfaceDescriptor UPDATE_AI = new UpdateAIInterfaceDescriptor();
	public static final class UpdateAIInterfaceDescriptor extends InterfaceDescriptor {
		public static final ServletInterfaceParameter AUTHORIZED_ISSUER = new ServletInterfaceParameter("A", "authorized issuer");
		public static final ServletInterfaceParameter AUTHORIZED_ISSUER_KEY_DATA = new ServletInterfaceParameter("KEY", "authorized issuer's public key data");
		public static final ServletInterfaceParameter LOWER_EMISSION_LIMIT = new ServletInterfaceParameter("L", "lower emission limit");
		public static final ServletInterfaceParameter UPPER_EMISSION_LIMIT = new ServletInterfaceParameter("H", "upper emission limit");
		public static final ServletInterfaceParameter ISSUER = new ServletInterfaceParameter("K", "issuer");
		public UpdateAIInterfaceDescriptor() {
			super(TransactionType.UPDATE_AI, "/updateAI", "update an authorized issuer's properties", SignatureRequirement.NONE, SecurityRequirement.NONE, createParameterList(AUTHORIZED_ISSUER,ISSUER), createParameterList(AUTHORIZED_ISSUER_KEY_DATA,LOWER_EMISSION_LIMIT,UPPER_EMISSION_LIMIT));
		}
	}
	
/*
 * 	
	OBLIGATIONS_SPLIT,
 * */	
	
	private static final Map<TransactionType, InterfaceDescriptor> allowedTransactions = new HashMap<TransactionType, InterfaceDescriptor>();
	private static String interfaceDescription;
	static {
		allowedTransactions.put(TransactionType.GET_TEMPLATE, GET_TEMPLATE);
		allowedTransactions.put(TransactionType.GET_PUBLIC_KEY, GET_PUBLIC_KEY);
		allowedTransactions.put(TransactionType.GET_BALANCE, GET_BALANCE);
		allowedTransactions.put(TransactionType.EMIT, EMIT);
		allowedTransactions.put(TransactionType.INFO, INFO);
		allowedTransactions.put(TransactionType.EXCHANGE, EXCHANGE);
		allowedTransactions.put(TransactionType.INVALIDATE, INVALIDATE);
		allowedTransactions.put(TransactionType.SPLIT, SPLIT);
		allowedTransactions.put(TransactionType.OBLIGATIONS_MERGE, OBLIGATIONS_MERGE);
		allowedTransactions.put(TransactionType.UPDATE_AI, UPDATE_AI);
		allowedTransactions.put(TransactionType.GET_INTERFACES, GET_INTERFACES);
		allowedTransactions.put(TransactionType.SECURE_MESSAGE, SECURE_MESSAGE);
		
		interfaceDescription = createInterfaceDescription();
	}
	
	public static String getInterfaceDescription() {
		return interfaceDescription;
	}
	
	private static final String createInterfaceDescription() {
		
		StringBuffer buf = new StringBuffer();
		
		Iterator<InterfaceDescriptor> descIter = allowedTransactions.values().iterator();
		while(descIter.hasNext()) {
			InterfaceDescriptor desc = descIter.next();
			
			buf.append(desc.getTransactionType().toString() + " mapped to " + desc.getRelativeURL() + " Parameters: " + createParameterDescription(desc) + "\n");
		}
		
		return buf.toString();
	}
	
	private static String createParameterDescription(InterfaceDescriptor desc) {
		StringBuffer buf = new StringBuffer();
		
		Iterator<ServletInterfaceParameter> iter = desc.getParameters().iterator();
		while(iter.hasNext()) {
			ServletInterfaceParameter param = iter.next();
			buf.append(param.getName() + "  (" + param.getDescription() + ")");
			
			if (iter.hasNext() || !desc.getOptionalParameters().isEmpty())
				buf.append(", ");
		}
		
		if(!desc.getOptionalParameters().isEmpty())
			buf.append("[");
		
		iter = desc.getOptionalParameters().iterator();
		while(iter.hasNext()) {
			ServletInterfaceParameter param = iter.next();
			buf.append(param.getName() + "  (" + param.getDescription() + ")");
			
			if (iter.hasNext())
				buf.append(", ");
		}

		if(!desc.getOptionalParameters().isEmpty())
			buf.append("]");
		
		return buf.toString();
	}
}
