/*******************************************************************************
 * Copyright (c) 2006-2010 Janis Schuller, nachtlicht-media
 * 
 * The software and the associated documentation are licensed under the 
 * GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007. For a detailed
 * license description, see LICENSE.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *******************************************************************************/
package de.nachtlicht_media.epoint.redcent.sqlImplementation;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import de.nachtlicht_media.epoint.redcent.exception.DatabaseErrorException;

public class JndiDataSourceFactory
{
	final String jndiName;
	
	public JndiDataSourceFactory(String jndiName)
	{
		this.jndiName = jndiName;
	}

	public DataSource getDataSource() throws DatabaseErrorException
	{
		try
		{
			Context ctx = new InitialContext();
			if(ctx == null) 
				throw new DatabaseErrorException("No initial naming context found. Cannot initialize datasource.");

			DataSource ds = (DataSource)ctx.lookup("java:/" + jndiName);
			return ds;
		}
		catch(NamingException nEx)
		{
			throw new DatabaseErrorException(nEx);
		}
	}
}
