/*******************************************************************************
 * Copyright (c) 2006-2010 Janis Schuller, nachtlicht-media
 * 
 * The software and the associated documentation are licensed under the 
 * GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007. For a detailed
 * license description, see LICENSE.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *******************************************************************************/
package de.nachtlicht_media.epoint.redcent.util;

import java.util.LinkedHashMap;
import java.util.Map;

import de.nachtlicht_media.epoint.redcent.exception.ConflictException;

public class SimpleTemplateParser
{
	public class Variable
	{
		String name;
		boolean isBraced = false;
		
		public Variable(String name)
		{
			this.name = name;
		}
		
		public Variable(String name, boolean isBraced)
		{
			this(name);
			this.isBraced = isBraced;
		}
		
		public String toString()
		{
			return "["+(isBraced?"b":"")+name+"]";
		}
		
		public boolean isBraced()
		{
			return isBraced;
		}
	}
	
	private String message;
	
	public SimpleTemplateParser(String message)
	{
		this.message = message;
	}
	
	public Map parseVariableNames() throws ConflictException
	{
		return parseVariableNames(0, new LinkedHashMap());
	}
	
	protected Map parseVariableNames(int startIndex, Map variables) throws ConflictException
	{
//		System.out.println("parseVariableNames("+startIndex+", "+variables+")");
		
		if (startIndex >= message.length())
			return variables;
			
		switch(message.charAt(startIndex))
		{
			case '$':
			{
//				System.out.print("Start of variable name at " + startIndex );
				Variable nextVariable = parseVariableName(message, ++startIndex);
				variables.put(nextVariable.name, nextVariable);
//				System.out.println(" --> " + name);
				return parseVariableNames(startIndex + nextVariable.name.length(), variables);
			}
			case '{':
			{
				// Verify that the next token is a variable name and allow optional Parameters to be passed
//				System.out.print("Start of brace " + startIndex);
				if (message.charAt(startIndex +1 ) == '$')
				{
					// skip first curly brace
					startIndex++;
					
					// parse variable name
					Variable name = parseVariableName(message, ++startIndex);
					name.isBraced = true;
					variables.put(name.name, name);
					startIndex = startIndex + name.name.length();
					// proceed to next closing curly brace
					while(message.charAt(startIndex++) != '}')
					{
						if (message.charAt(startIndex) == '$')
							throw new ConflictException("Illegal start of expression $ at " + startIndex);
					}
					return parseVariableNames(startIndex++, variables);
				}
				else
					throw new ConflictException("Illegal expression at index " + startIndex);
			}
			default:
				return parseVariableNames(++startIndex, variables);
		}
	}

	private Variable parseVariableName(String message, int startIndex) throws ConflictException
	{
		// Parse one variable name and proceed
		if (message.charAt(startIndex) == '{')
		{
			// Parse variable name enclosed by curly braces
			return parseEnclosedName(message, startIndex);
		}
		else
		{
			// Parse default variable name
			return parseName(message, startIndex);
		}
	}

	private Variable parseEnclosedName(String message, int startIndex) throws ConflictException
	{
		String name = String.valueOf(message.charAt(startIndex));
		
		while (message.charAt(startIndex++) != '}')
		{
			if (!(Character.isJavaIdentifierPart(message.charAt(startIndex)) || message.charAt(startIndex) == '}' || message.charAt(startIndex) == ' '))
				throw new ConflictException("Illegal character found at " + startIndex + ": " + message.charAt(startIndex));
			
			name += message.charAt(startIndex);
		}
		
//		System.out.println("Found enclosed name: " + name);
		
		return new Variable(name);
	}

	private Variable parseName(String message, int startIndex)
	{
		String name = String.valueOf(message.charAt(startIndex++));
		
		while (Character.isJavaIdentifierPart(message.charAt(startIndex)) && message.charAt(startIndex) != '$')
		{
			name += message.charAt(startIndex++);
		}
		
//		System.out.println(" ( Found simple name: " + name + ") ");
		
		return new Variable(name);
	}
	
	public static void main(String[] args) throws Exception
	{
		String message = "This is the bloody message with $COOL identifier. Some ${OTHER}$NEXT is way $cooler. This is {$TODO, date}";
		SimpleTemplateParser parser = new SimpleTemplateParser(message);
		
		Map names = parser.parseVariableNames();
		System.out.println("------\nFound variables: " + names);
	}
}
