/*******************************************************************************
 * Copyright (c) 2006-2007 Janis Schuller, Nachtlicht-Media
 *
 * The software and the associated documentation are licensed under the 
 * GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007. For a detailed
 * license description, see LICENSE.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *******************************************************************************/
package hu.vems.epoint.walcent.servlet;

import java.io.*;
import java.net.*;

import javax.naming.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.sql.*;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement; 

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.apache.commons.codec.net.*;
import org.apache.commons.logging.*;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;

import hu.vems.epoint.walcent.*;
import hu.vems.epoint.walcent.interfaces.*;
import hu.vems.epoint.walcent.servlet.filter.*;
import hu.vems.epoint.walcent.sqlImplementation.*;
import hu.vems.epoint.walcent.util.FPRUtils;
import hu.vems.epoint.walcent.servlet.crypto.MarketServletMessageDecryptionVerificationFactory;
import de.nachtlicht_media.epoint.redcent.*;
import de.nachtlicht_media.epoint.redcent.exception.*;
import de.nachtlicht_media.epoint.redcent.interfaces.*;
import de.nachtlicht_media.epoint.redcent.servlet.exception.*;
import de.nachtlicht_media.epoint.redcent.sqlImplementation.*;

/**
 * The WalCentServlet is currently used to provide the http(s) interface to the WalCent application. During
 * its initialization phase creates the WalCentServlet the necessary instance of the WalCent application.
 * @author js
 *
 */
public class WalCentServlet extends HttpServlet
{
	private static final Log log = LogFactory.getLog(WalCentServlet.class);
	static protected WalCent market;
	
	public WalCentServlet()
	{
	}

	/**
	 * Init method for servlet. If initialization of issuer reference fails, a ServletException is thrown
	 * to prevent request processing until the servlet is re-deployed and the issuer is correctly initialized.
	 * 
	 * @see javax.servlet.GenericServlet#init(javax.servlet.ServletConfig)
	 */
	public void init(final ServletConfig config) throws ServletException
	{
		super.init(config);
		
		log.info("initializing WalCentServlet");
		
		WebApplicationContext ctx = 
			WebApplicationContextUtils.getRequiredWebApplicationContext(
				this.getServletContext());
		market = (WalCent) ctx.getBean("walcent");

		log.info("...done.");
	}
				
	/**
	 * Use a JNDI datasource to store data.
	 */
/*	private DataSource  src  = null;
	protected DataSource getDataSource() throws DatabaseErrorException
	{
		if (src == null)
		{
			try
			{
				String dataSourceJndiName = config.getServletContext().getInitParameter("DataSourceJndiName");

				Context ctx = new InitialContext();
				if(ctx == null) 
					throw new DatabaseErrorException("No initial naming context found. Cannot initialize datasource.");

				DataSource ds = (DataSource)ctx.lookup("java:/" + dataSourceJndiName);
				return ds;
			}
			catch(NamingException nEx)
			{
				throw new DatabaseErrorException(nEx);
			}
		}

		return src;
	}*/
			

	/**
	 * Redirect GET-request to doPost to be processed. If a certain URL shall not be available via
	 * GET-requests, a servlet filter has to be used to redirect these request before entering the
	 * servlet's code.
	 */
	protected void doGet(HttpServletRequest arg0, HttpServletResponse arg1) throws ServletException, IOException
	{
		doPost(arg0, arg1);
	}
	
	/**
	 * Process POST-requests: wraps http requests to method calls to the market application.
	 * @see javax.servlet.http.HttpServlet#doPost(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
	 */
	protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException
	{
		log.trace("Processing request " + req.getRequestURI() + " with parameters " + req.getParameterMap());
		
		String sp=req.getServletPath();		
		
		if ("/sql".equals(sp))
		{
			String str = req.getParameter("S"); 
			
	        PrintWriter out = resp.getWriter();
	        out.println("S='"+str+"'"); 		

	        try {
				
				Context ctx = new InitialContext();

				out.println("got Context"); 		

				DataSource ds = (DataSource)ctx.lookup("java:/comp/env/jdbc/postgres"); 

				out.println("got DataSource"); 		
				
				Connection con =  ds.getConnection();

		        out.println("connected"); 		
				
		        PreparedStatement pStm = null;
				ResultSet rs = null;

//				pStm = con.prepareStatement("CREATE TABLE proba ( id smallint DEFAULT -1 NOT NULL, str character varying(64) DEFAULT 'UNDEFINED'::character varying NOT NULL);");			
//				pStm = con.prepareStatement("INSERT INTO proba VALUES (3,'hrom')");			
//				pStm = con.prepareStatement("SELECT DISTINCT * FROM proba");			
				pStm = con.prepareStatement(str);			
				
//				rs = pStm.executeQuery();

				try {
					rs=pStm.executeQuery();
				} catch (Exception ex) {}

		        out.println("executed"); 		

		        if (rs!=null) {
		        	while (rs.next()) {
		        		//rs.next();
		        		//int i = rs.getInt(1);
		        		String s=rs.getString(1);
		        		String s2=rs.getString(2);
		        		out.println(s+" "+s2);
		        	};
		        }
			
		        out.println("END"); 		

		        pStm = null;
				rs = null;
			}	
			catch (Exception ex)
			{
				int i=0;
				// Aquire a ProcessingException for the thrown exception to be able
				// to return the correct response status and short text
//				ProcessingException pEx = getProcessingExceptionMapper().getMappedException(ex);
//				respondWithProcessingException(resp, pEx);
			} 			
		}
		else if ("/pubkey.asc".equals(sp))
		{
			doPubkey(resp);
		}
		else if ("/keyfpr.asc".equals(sp))
		{
			doKeyFPR(resp);
		}
		else if ("/template".equals(sp))
		{
			doTemplate(req, resp);
		}
		else if ("/service".equals(sp))
		{
			doService(resp);
		}
		else if ("/account".equals(sp))
		{
			doAccount(req, resp);
		}
		else if ("/withdraw".equals(sp))
		{
			doWithdraw(req, resp);
		}
		else if ("/offer".equals(sp))
		{
			doOffer(req, resp);
		}
		else if ("/search".equals(sp))
		{
			doSearch(req, resp);
		}
/*		else if ("/info".equals(sp))
		{
			doInfo(resp);
		}
*/
		else
		{
			try {
				super.doPost(req, resp);
			}
			catch (Exception ex)
			{
				ProcessingException pEx = getProcessingExceptionMapper().getMappedException(ex);
				respondWithProcessingException(resp, pEx);
			}
		}
	}

	private void doSearch(HttpServletRequest req, HttpServletResponse resp) throws IOException
	{
		try
		{
			
			byte[] IssuerAFPR = FPRUtils.parseFPR(req.getParameter("IA"));
			byte[] IssuerBFPR = FPRUtils.parseFPR(req.getParameter("IB"));

			String RAText = req.getParameter("RA");
			long RA = -1;
			if (RAText!=null) RA = Long.parseLong(RAText);
			
			String RBText = req.getParameter("RB");
			long RB = -1;
			if (RBText!=null) RB = Long.parseLong(RBText);

			String VAText = req.getParameter("VA");
			long VA = -1;
			if (VAText!=null) VA = Long.parseLong(VAText);
			
			String VBText = req.getParameter("VB");
			long VB = -1;
			if (VBText!=null) VB = Long.parseLong(VBText);
			
			if ((IssuerAFPR==null) || (IssuerBFPR==null))
				throw new MalformedRequestException("Fatal Error: offered or requested currency not specified");
			
//			if (((RA<=0) || (RB<=0)) && (VA>0) && (VB>0))
//				throw new MalformedRequestException("Fatal Error: neither min. exchange ratio nor value limit are specified");
				
			// get cert for rest, make targetDist valid with amount
			byte[] newCert = market.search(IssuerAFPR, IssuerBFPR, RA, RB, VA, VB);

			resp.setStatus(200);
			resp.getOutputStream().write(newCert);
					
		}
		catch(Exception ex)
		{
			// Aquire a ProcessingException for the thrown exception to be able
			// to return the correct response status and short text
			ProcessingException pEx = getProcessingExceptionMapper().getMappedException(ex);
			respondWithProcessingException(resp, pEx);
		}
		
	}

	private void doOffer(HttpServletRequest req, HttpServletResponse resp) throws IOException
	{
		try
		{
			if (req.getParameterMap().isEmpty())
			{
				throw new MalformedRequestException("Illegal call without arguments");
			}
			else if (req.getAttribute(SecureMessageRedirectionFilter.ATTR_SECURE_MESSAGE_HANDLER) == null)
			{
				throw new MalformedRequestException("Illegal call without secure message handler.");
			}
			else
			{
				// This attribute has to be set by SecureMessageRedirectionFilter that validates/decrypts and redirects messages
				MessageDecryptionVerificationHandler msg = (MessageDecryptionVerificationHandler)req.getAttribute(SecureMessageRedirectionFilter.ATTR_SECURE_MESSAGE_HANDLER);
				
				byte[] UserFPR = FPRUtils.parseFPR(req.getParameter("U"));
				
				if (!msg.isValid().booleanValue())
					throw new MalformedRequestException("Illegal call to /offer with non-valid message.");
				else if (msg.getSignersFingerprint() == null)
					throw new MalformedRequestException ("Illegal call: requests to /offer have to be signed.");
				else if (!msg.isClearText().booleanValue())
					throw new BadsignatureException ("Illegal call: requests to /offer must be clearsigned.");
				else if (!FPRUtils.compareFPR(FPRUtils.parseFPR(msg.getSignersFingerprint()), UserFPR))
					throw new BadsignatureException ("Illegal call: requests to /offer must be signed with account holder's key.");
				else
				{
					// msg is valid and signed

					if (UserFPR == null)
						throw new MalformedRequestException ("Illegal call: requests to /offer have to contain a valid user key fingerprint");
					
					byte[] IssuerAFPR = FPRUtils.parseFPR(req.getParameter("IA"));
					byte[] IssuerBFPR = FPRUtils.parseFPR(req.getParameter("IB"));

					String RAText = req.getParameter("RA");
					long RA = -1;
					if (RAText!=null) RA = Long.parseLong(RAText);
					
					String RBText = req.getParameter("RB");
					long RB = -1;
					if (RBText!=null) RB = Long.parseLong(RBText);

					String VAText = req.getParameter("VA");
					long VA = -1;
					if (VAText!=null) VA = Long.parseLong(VAText);
					
					String VBText = req.getParameter("VB");
					long VB = -1;
					if (VBText!=null) VB = Long.parseLong(VBText);
					
					String SNText = req.getParameter("PSN");
					long SN = -1;
					if (SNText!=null) SN = Long.parseLong(SNText);
					
					String reqText = new String(msg.getMessage(),"UTF-8");
					
					if ((IssuerAFPR!=null) || (SN==-1)) {
					
						if ((IssuerAFPR==null) || (IssuerBFPR==null))
							throw new MalformedRequestException("Fatal Error: offered or requested currency not specified");
						
						if ((RA<=0) || (RB<=0))
							throw new MalformedRequestException("Fatal Error: offer exchange ratio not specified");

						if ((VA>0) && (VB>0))
							throw new MalformedRequestException("Fatal Error: both VA and VB are specified");
						
					} else {

						if (IssuerBFPR!=null) throw new MalformedRequestException("Unexpected 'IB' parameter for deactivate offer command");
						if (RA!=-1) throw new MalformedRequestException("Unexpected 'RA' parameter for deactivate offer command");
						if (RB!=-1) throw new MalformedRequestException("Unexpected 'RB' parameter for deactivate offer command");
						if (VA!=-1) throw new MalformedRequestException("Unexpected 'VA' parameter for deactivate offer command");
						if (VB!=-1) throw new MalformedRequestException("Unexpected 'VB' parameter for deactivate offer command");
						
					}
					
					// get cert for rest, make targetDist valid with amount
					byte[] newCert = market.offer(UserFPR, IssuerAFPR, IssuerBFPR, RA, RB, VA, VB, SN, reqText);

					resp.setStatus(200);
					resp.getOutputStream().write(newCert);
				}
			}
		}
		catch(Exception ex)
		{
			// Aquire a ProcessingException for the thrown exception to be able
			// to return the correct response status and short text
			ProcessingException pEx = getProcessingExceptionMapper().getMappedException(ex);
			respondWithProcessingException(resp, pEx);
		}
		
	}

	
	private void doAccount(HttpServletRequest req, HttpServletResponse resp) throws IOException {

		try
		{
			// obligation certificate -> user account deposit request
			String cert = req.getParameter("C");
			String pubKey = req.getParameter("KEYDATA");
			
			if (cert!=null) {
				
				if (cert.length() == 0)
					throw new MalformedRequestException("Illegal value for parameter 'C'. Only a valid obligation certificate will be accepted.");

//				if (pubKey == null || pubKey.length() == 0)
//					throw new MalformedRequestException("Illegal value for parameter 'KEYDATA'. Only a valid, signed draft combined with the corresponding public key will be processed.");

				MessageDecryptionVerificationHandler signedMessage = MarketServletMessageDecryptionVerificationFactory.getMessageHandler();
				
				// add the public key to use for verification
				if (pubKey != null && pubKey.length() != 0) signedMessage.addTemporaryPublicKey(pubKey);
				
				signedMessage.setMessage(cert.getBytes("UTF-8"));
				signedMessage.analyse();
				
				if (signedMessage.isValid() == null)
					throw new ProcessingException("The Decryption/Verification-Handler could not validate message. Request ignored.");
				else if (signedMessage.isValid().booleanValue()) 
				{
					// message could be processed and signature is valid
					// parse draft and start processing if all parameters can be found
					String messageContent = signedMessage.getClearText();
					
					// extract clear text from signedMessage
					if (messageContent != null && messageContent.length() > 0)
					{
						byte[] newCert = market.processDepositMessage(signedMessage);
						
						if (newCert != null)
						{
							resp.setStatus(200);
							resp.getOutputStream().write(newCert);
						}
						else
							throw new ConflictException("Deposit request could not be processed.");
					}
					else
						throw new MalformedRequestException("Illegal value for parameter 'D', empty message content.");
				}
				else
					throw new ProcessingException("The signature of the certificate is not valid. Request ignored.");
			}
			else
			{
				byte[] UserFPR = FPRUtils.parseFPR(req.getParameter("U"));
				
				if (UserFPR==null)
					throw new MalformedRequestException("Illegal value for parameter 'U'. No valid user key fingerprint specified.");
				
				byte[] newCert = market.processAccountInfo(UserFPR);
				
				if (newCert != null)
				{
					resp.setStatus(200);
					resp.getOutputStream().write(newCert);
				}
				else
					throw new ConflictException("Deposit request could not be processed.");
				
			}
			
		}
		catch (Exception ex)
		{
			// Aquire a ProcessingException for the thrown exception to be able
			// to return the correct response status and short text
			ProcessingException pEx = getProcessingExceptionMapper().getMappedException(ex);
			respondWithProcessingException(resp, pEx);
		}
		
	}
	
	private void doWithdraw(HttpServletRequest req, HttpServletResponse resp) throws IOException
	{
		try
		{
			if (req.getParameterMap().isEmpty())
			{
				throw new MalformedRequestException("Illegal call without arguments");
			}
			else if (req.getAttribute(SecureMessageRedirectionFilter.ATTR_SECURE_MESSAGE_HANDLER) == null)
			{
				throw new MalformedRequestException("Illegal call without secure message handler.");
			}
			else
			{
				// This attribute has to be set by SecureMessageRedirectionFilter that validates/decrypts and redirects messages
				MessageDecryptionVerificationHandler msg = (MessageDecryptionVerificationHandler)req.getAttribute(SecureMessageRedirectionFilter.ATTR_SECURE_MESSAGE_HANDLER);
				
				byte[] UserFPR = FPRUtils.parseFPR(req.getParameter("U"));
				
				if (!msg.isValid().booleanValue())
					throw new MalformedRequestException("Illegal call to /withdraw with non-valid message.");
				else if (msg.getSignersFingerprint() == null)
					throw new MalformedRequestException ("Illegal call: requests to /withdraw have to be signed.");
				else if (!msg.isClearText().booleanValue())
					throw new BadsignatureException ("Illegal call: requests to /withdraw must be clearsigned.");
				else if (!FPRUtils.compareFPR(FPRUtils.parseFPR(msg.getSignersFingerprint()), UserFPR))
					throw new BadsignatureException ("Illegal call: requests to /withdraw must be signed with account holder's key.");
				else
				{
					// msg is valid and signed

					if (UserFPR == null)
						throw new MalformedRequestException ("Illegal call: requests to /withdraw have to contain a valid user key fingerprint");
					
					byte[] IssuerFPR = FPRUtils.parseFPR(req.getParameter("I"));

					if (IssuerFPR == null)
						throw new MalformedRequestException ("Illegal call: requests to /withdraw have to contain a valid issuer key fingerprint");
		
					String targetDigest = req.getParameter("D");
					
					if (targetDigest == null || targetDigest.trim().length() == 0)
						throw new MalformedRequestException ("Illegal call: requests to /withdraw have to contain non-empty parameter D.");
					
					String amountText = req.getParameter("V");

					long amount = 0;
					if (amountText!=null) amount = Long.parseLong(amountText);
					
					String reqText = new String(msg.getMessage(),"UTF-8");
					
					// just for fun: check if redirect parameters are the same as in the messageHandler
					// might differ when the ServletContainer messes sessions up.
					String decodedMessage = URLDecoder.decode(msg.getClearText(), "UTF-8");
					if(			decodedMessage.indexOf(targetDigest) == -1
							|| 	decodedMessage.indexOf(String.valueOf(amount)) == -1
					)
					{
						throw new MalformedRequestException("Fatal Error: MessageHandler and request parameters do not match.");
					}
					
					if(amount < 0)
						throw new MalformedRequestException("Fatal Error: The amount of ePoints to withdraw must not be less than zero.");
					
					// get cert for rest, make targetDist valid with amount
					byte[] newCert = market.withdraw(UserFPR, IssuerFPR, amount, targetDigest, reqText);

					resp.setStatus(200);
					resp.getOutputStream().write(newCert);
				}
			}
		}
		catch(Exception ex)
		{
			// Aquire a ProcessingException for the thrown exception to be able
			// to return the correct response status and short text
			ProcessingException pEx = getProcessingExceptionMapper().getMappedException(ex);
			respondWithProcessingException(resp, pEx);
		}
		
	}
	
	private void doPubkey(HttpServletResponse resp) throws FileNotFoundException, IOException
	{
		resp.setContentType("text/plain");
		resp.setStatus(200);
		
		BufferedReader keyReader = new BufferedReader(new FileReader(getServletContext().getRealPath("/pubkey.asc")));
		
		String line = null;
		while((line = keyReader.readLine()) != null)
			resp.getWriter().println(line);
		
		keyReader.close();
	}

	private void doKeyFPR(HttpServletResponse resp) throws IOException
	{
		resp.setContentType("text/plain");
		resp.setStatus(200);
		
		try
		{
			resp.getWriter().println(market.getMarketFingerPrintText());
		}
		catch (Exception ex)
		{
			ProcessingException pEx = getProcessingExceptionMapper().getMappedException(ex);
			respondWithProcessingException(resp, pEx);
		}
	}

	private void doTemplate(HttpServletRequest req, HttpServletResponse resp) throws IOException
	{
		if (req.getParameter("ID") != null)
		{
			String id = req.getParameter("ID").toUpperCase();

			log.trace("Getting template for id " + id);

			try
			{
				String template = market.getTemplateManager().getTemplate(id);

				if (template != null)
				{
					resp.setContentType("text/plain; ; charset=utf-8");
					resp.setStatus(200);
					resp.getOutputStream().write(template.getBytes("UTF-8"));
				}
				else
					throw new NotFoundException("A template identified by hash value " + id + " could not be found.");
			}
			catch(Exception ex)
			{
				// Aquire a ProcessingException for the thrown exception to be able
				// to return the correct response status and short text
				ProcessingException pEx = getProcessingExceptionMapper().getMappedException(ex);
				respondWithProcessingException(resp, pEx);
			}
		}
		else
			respondWithProcessingException(resp, getProcessingExceptionMapper().getMappedException(new MalformedRequestException("No parameter ID specified.")));
	}

	private void doService(HttpServletResponse resp) throws IOException
	{
		try
		{
			resp.setContentType("text/plain; ; charset=utf-8");
			resp.setStatus(200);

			resp.getOutputStream().write(market.getDeployedServerServiceContract());
		}
		catch(Exception ex)
		{
			// Aquire a ProcessingException for the thrown exception to be able
			// to return the correct response status and short text
			ProcessingException pEx = getProcessingExceptionMapper().getMappedException(ex);
			respondWithProcessingException(resp, pEx);
		}
	}

/*	private void doInfo(HttpServletRequest req, HttpServletResponse resp) throws IOException
	{
		if (req.getParameterMap().isEmpty())
		{
			try
			{
				byte[] signedServerStatus = market.getServerStatus();

				resp.setContentType("text/plain");
				resp.setStatus(200);
				resp.getOutputStream().write(signedServerStatus);
			}
			catch(Exception ex)
			{
				// Aquire a ProcessingException for the thrown exception to be able
				// to return the correct response status and short text
				ProcessingException pEx = getProcessingExceptionMapper().getMappedException(ex);
				respondWithProcessingException(resp, pEx);
			}
		}
		else
		{
			try
			{
				if (req.getParameterMap().get("ID") != null || req.getParameterMap().get("KEY") != null )
				{
					String id = req.getParameter("ID");
					
					if (id == null)
						id = req.getParameter("KEY");
					
					EpointRecord rec = market.findValidRecordByID(id.toUpperCase());
					
					if (rec != null)
					{
						resp.getOutputStream().write(rec.getCert());
					}
					else
						throw new NotFoundException("Not found");
				}
				else
					throw new MalformedRequestException("Invalid request: either no parameter or exactly one of 'ID' or 'KEY' has to be specified.");
			}
			catch(Exception ex)
			{
				ProcessingException pEx = getProcessingExceptionMapper().getMappedException(ex);
				respondWithProcessingException(resp, pEx);
			}
		}
		
//			debug(req, resp);
	}*/
	
	private String checkTargetMarket(String targetMarket) throws MalformedRequestException, ConflictException
	{
		if(targetMarket == null)
			throw new MalformedRequestException("Fatal Error: No parameter 'K' submitted containing the market's fingerprint.");
		else
		{
			targetMarket = targetMarket.toUpperCase().replaceAll(" ","");
			String thisMarket = market.getMarketFingerPrintText().toUpperCase().replaceAll(" ", "");
			
			if (!targetMarket.equals(thisMarket))
				throw new MalformedRequestException("Fatal Error: The submitted markets fingerprint("+targetMarket+") does not match the fingerprint of this market ("+thisMarket+").");
		}
		return targetMarket;
	}

	protected void respondWithProcessingException(HttpServletResponse resp, ProcessingException mappedException) throws IOException
	{
		// is forwarded to the container's default error handler
		resp.sendError(mappedException.getStatus(), mappedException.getMessage());
	}

	protected ProcessingExceptionMapper defaultExceptionMapper;
	protected ProcessingExceptionMapper getProcessingExceptionMapper()
	{
		if (defaultExceptionMapper == null)
			defaultExceptionMapper = new DefaultProcessingExceptionMapper();
		
		return defaultExceptionMapper;
	}

	public String getServletInfo()
	{
		return "WalCent application gateway servlet.";
	}
}
