package hu.vems.epoint.walcent.servlet.crypto;

import java.io.FileInputStream;

import javax.servlet.ServletContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.bouncycastle.openpgp.PGPPublicKeyRing;
import org.bouncycastle.openpgp.PGPPublicKeyRingCollection;
import org.bouncycastle.openpgp.PGPSecretKeyRingCollection;
import org.bouncycastle.openpgp.PGPUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.context.ServletContextAware;
import org.springframework.web.context.support.WebApplicationContextUtils;

import hu.vems.epoint.walcent.*;
import de.nachtlicht_media.epoint.redcent.crypto.BouncycastleMessageDecryptionVerificationHandler;
import de.nachtlicht_media.epoint.redcent.interfaces.AuthorizedIssuerStore;
import de.nachtlicht_media.epoint.redcent.interfaces.MessageDecryptionVerificationHandler;
import de.nachtlicht_media.epoint.redcent.servlet.RedCentServletConfig;
import de.nachtlicht_media.epoint.redcent.servlet.crypto.*;

public class MarketServletMessageDecryptionVerificationFactory implements ServletContextAware 
{
	private static final Log log = LogFactory.getLog(MarketServletMessageDecryptionVerificationFactory.class);
	private ServletContext context;
	
	@Autowired
	private WalCent walcent;

	/** 
	 * Don't allow instances of this class. Only calls to static methods are allowed.
	 *
	 */
	private MarketServletMessageDecryptionVerificationFactory()
	{
	}

	public void setServletContext(ServletContext context)
	{
		log.info("ServletContext " + context + " was injected.");
		this.context = context;
	}

	private static PGPPublicKeyRingCollection issuersPublicKey;
	private static PGPSecretKeyRingCollection issuersSecretKey;
	private static String issuersPrivateKeyPassword;
	private static PGPPublicKeyRingCollection trustedPublicKeys;
	private static PGPPublicKeyRing masterKey;
	private static AuthorizedIssuerStore authorizedIssuerStore;
	
	public void init() throws Exception
	{
		if (issuersPublicKey != null)
			throw new Exception("ServletMessageDecryptionVerificationFactory is already initialized!");
		
		issuersPublicKey = new PGPPublicKeyRingCollection(PGPUtil.getDecoderStream(new FileInputStream(context.getRealPath(context.getInitParameter("IssuersPublicKeyRingPath")))));
		issuersSecretKey = new PGPSecretKeyRingCollection(PGPUtil.getDecoderStream(new FileInputStream(context.getRealPath(context.getInitParameter("IssuersPrivateKeyRingPath")))));
		issuersPrivateKeyPassword = context.getInitParameter("IssuersPrivateKeyPassword");
		trustedPublicKeys = new PGPPublicKeyRingCollection(PGPUtil.getDecoderStream(new FileInputStream(context.getRealPath(context.getInitParameter("TrustedPublicKeyRingPath")))));
		
		masterKey = null;
		
		authorizedIssuerStore = null;
	}

	public static synchronized MessageDecryptionVerificationHandler getMessageHandler() throws Exception
	{
		return new BouncycastleMessageDecryptionVerificationHandler(authorizedIssuerStore, issuersPublicKey, issuersSecretKey, issuersPrivateKeyPassword, masterKey, trustedPublicKeys);
	}
}
