/*******************************************************************************
 * Copyright (c) 2006-2007 Janis Schuller, Nachtlicht-Media
 * 
 * The software and the associated documentation are licensed under the 
 * GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007. For a detailed
 * license description, see LICENSE.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *******************************************************************************/
package hu.vems.epoint.walcent.sqlImplementation;

import java.io.*;
import java.lang.ref.*;
import java.math.BigInteger;
import java.security.*;
import java.sql.*;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.Date;

import javax.servlet.http.*;
import javax.sql.*;

import org.apache.commons.codec.*;
import org.apache.commons.codec.binary.*;
import org.apache.commons.logging.*;
import org.springframework.beans.factory.annotation.Autowired;

import hu.vems.epoint.walcent.*;
import hu.vems.epoint.walcent.sqlImplementation.*;
import hu.vems.epoint.walcent.util.*;
import hu.vems.epoint.walcent.interfaces.*;

import de.nachtlicht_media.epoint.redcent.*;
import de.nachtlicht_media.epoint.redcent.crypto.BouncycastleUtil;
import de.nachtlicht_media.epoint.redcent.exception.*;
import de.nachtlicht_media.epoint.redcent.interfaces.*;
import de.nachtlicht_media.epoint.redcent.messaging.*;
import de.nachtlicht_media.epoint.redcent.sqlImplementation.*;
import de.nachtlicht_media.epoint.redcent.util.*;
import de.nachtlicht_media.epoint.redcent.util.StringUtils;

/**
 * The main WalCent implementation based on a postgresql database.
 * @author js, fzs
 *
 */
public class WalCentSqlImplementation extends WalCent
{
	public static class StatRecord
	{
		public static final String AC_APP_EXCHANGE = "APP_EXCHANGE";

		public static final String AC_CREATE_RECORD = "CREATE_RECORD";
		public static final String AC_FIND_VALID_RECORD = "FIND_VALID_RECORD";
		public static final String AC_FIND_INVALID_RECORD = "FIND_INVALID_RECORD";
		public static final String AC_VALIDATE_RECORD = "VALIDATE_RECORD";
		public static final String AC_COMMIT = "COMMIT";
		public static final String AC_LOCK = "LOCK";
		public static final String AC_FIND_USERACC_RECORD = "FIND_USERACC_RECORD";
		public static final String AC_SET_USERACC_RECORD = "SET_USERACC_RECORD";
		
		public String action;
		public String type;
		public long duration;
		
		public StatRecord(String action, String type, long duration)
		{
			super();
			this.action = action;
			this.type = type;
			this.duration = duration;
		}
		
		public String getData()
		{
			return action+";"+type+";"+duration;
		}
	}
	
	private static final boolean COLLECT_STATS = false;
	private StatsCollector statsCollector;
	public class StatsCollector extends ArrayList
	{
		public void logStatRecord(WalCentSqlImplementation.StatRecord rec)
		{
			if (COLLECT_STATS && rec != null)
				add(rec);
		}
	}

	private static final Log log = LogFactory.getLog(WalCentSqlImplementation.class);

	public WalCentSqlImplementation(RedCentConfig config)
	{
		super(config);
	}

	@Autowired
	private DataSource dataSource  = null;	
	
	@Autowired
	private MarketSQLDB marketDB;
	
	protected MarketSQLDB getMarketDB() throws DatabaseErrorException
	{
		if(marketDB == null)
		{
			marketDB = new MarketSQLDB();
			
			if (COLLECT_STATS)
			{
				if (statsCollector == null)
					statsCollector = new StatsCollector();
				
				marketDB.setStatsCollector(statsCollector);
			}
		}
		
		return marketDB;
	}
	protected ArrayList getDBStats() throws DatabaseErrorException
	{
		return statsCollector;
	}
	
/*	protected DataSource getDataSource() throws DatabaseErrorException
	{
		return null;
	}*/

/*	// use a template manager that stores revisions to an sql database table
	protected void initTemplateManager() throws Exception
	{
		final String templateDir = getTemplateRealPath();
		log.debug("Template dir is: " + templateDir);
		
		templateManager = new SqlDbStoringTemplateManager(null)
		{
			protected String getTemplatePath()
			{
				return templateDir;
			}
		};
		
		templateManager.init(this);
	}*/
	
	/** (non-Javadoc)
	 * @see de.nachtlicht_media.epoint.walcent.WalCent#initDocumentStore()
	 */
/*	protected void initDocumentStore() throws Exception
	{
		documentStore = new SqlDBStoringDocumentStore(this);
		documentStore.init();
	}*/

	public boolean testDataSource() throws Exception
	{
		if (dataSource != null)
		{
			Connection c = getConnection();
			
			if (c == null)
				return false;
			else
			{
				log.info("Datasource bound, proceeding to test database integrity.");
				c.close();
			}
		}
		
		log.info("Testing EpointSQLDB...");
		Connection con = null;
		try
		{
			con = getConnection();
			getMarketDB().beginTransaction(con);
			getMarketDB().lockWrites(con,"Offer");
			
			getMarketDB().verifyDatabase(con);
		}
		catch (SQLException ex)
		{
			con.rollback();
			throw new DatabaseErrorException(ex);
		}
		finally
		{
			closeConnection(con);
		}
		log.info("Testing EpointSQLDB succeeded.");
		
		return true;
	}

/*	private void checkResultType(int resultType) throws MalformedRequestException
	{
		if (		resultType != WalCent.RETURN_RESULTING_CERT 
				&& 	resultType != WalCent.RETURN_TRANSACTION_RECEIPT
				&&	resultType != WalCent.RETURN_TRANSACTIONVALUE_STRING_AS_BYTEARRAY_UTF8)
		{
			throw new MalformedRequestException("Illegal return type ("+resultType+") specified for command.");
		}
	}

	protected void logTransaction(EpointRecord newRec)
	{
		try
		{
			if (xLog != null)
			{
				if (xLog.isInfoEnabled())
					xLog.info(getXLogConverter().convert(newRec));
			}
			else
				log.warn("Transaction logger not initialized!");
		}
		catch (Exception e)
		{
			log.error("Could not write to transaction log.", e);
		}
	}

	private String checkTargetDigest(String digestName, String targetID) throws MalformedRequestException
	{
		if (targetID == null || targetID.trim().length() == 0)
			throw new MalformedRequestException(digestName + " digest to emit must not be empty.");
		else
		{
			try
			{
				// Digests are always uppercase whithout spaces
				targetID = targetID.trim().toUpperCase().replaceAll(" ","");
				
				if (targetID.length() != 40)
					throw new MalformedRequestException(digestName + " digest must be HEX-decodable and have a length of 40 chars.");			

				Hex.decodeHex(targetID.toLowerCase().toCharArray());
				
				return targetID;
			}
			catch(DecoderException ex)
			{
				throw new MalformedRequestException(digestName + " digest must be HEX-decodable.");
			}
		}
	}*/

	private String checkKeyFingerprint(String fingerprint) throws MalformedRequestException
	{
		if (fingerprint == null || fingerprint.trim().length() == 0)
			throw new MalformedRequestException("Key fingerprint must not be empty.");
		else
		{
			try
			{
				// Digests are always uppercase whithout spaces
				fingerprint = fingerprint.trim().toUpperCase().replaceAll(" ","");
				
				if (fingerprint.length() != 40)
					throw new MalformedRequestException("Key fingerprint must be HEX-decodable and have a length of 40 chars.");			

				Hex.decodeHex(fingerprint.toLowerCase().toCharArray());
				
				return fingerprint;
			}
			catch(DecoderException ex)
			{
				throw new MalformedRequestException("Key fingerprint must be HEX-decodable.");
			}
		}
	}

	private String  checkTargetMarket(String targetMarket) throws RedcentProcessingException
	{
		if(targetMarket == null)
			throw new MalformedRequestException("No parameter targetMarket present containing the market's fingerprint.");
		else
		{
			targetMarket = targetMarket.toUpperCase().replaceAll(" ","");
			String thisMarket = getMarketFingerPrintText().toUpperCase().replaceAll(" ", "");
			
			if (!targetMarket.equals(thisMarket))
				throw new MalformedRequestException("The submitted markets fingerprint("+targetMarket+") does not match the fingerprint of this market ("+thisMarket+").");
		}
		
		return targetMarket;
	}
	
	public byte[] processAccountInfo(byte[] UserFPR) throws RedcentProcessingException, SQLException, UnsupportedEncodingException
	{
		String message = createAccountInfo(UserFPR, "");
		return signMessage(message);
	}
	
	public OfferPathRecord searchOfferPath(Connection con, byte[] IA, byte[] IB, long VA, long VB) throws Exception
	{

		OfferPathRecord best=null;
		
		OfferRecord od=getMarketDB().findBestOffer(con, IA, IB, VA, VB);
		
		if (od!=null) {
			
			OfferPathRecord op=new OfferPathRecord();
			op.offer=new OfferRecord[1];
			op.offer[0]=od;
			op.Calculate();
			
			best=op;
		}

		byte[] IM=getMarketDB().findBestIMCurrency(con, IA, IB);
		
		if (IM!=null) {
			OfferRecord oa=getMarketDB().findBestOffer(con, IA, IM, -1, -1);
			OfferRecord ob=getMarketDB().findBestOffer(con, IM, IB, -1, -1);
			if ((oa!=null) && (ob!=null)) {

				OfferPathRecord op=new OfferPathRecord();
				op=new OfferPathRecord();
				op.offer=new OfferRecord[2];
				op.offer[0]=ob;
				op.offer[1]=oa;
				op.Calculate();
				
				if ((best==null) || (op.R>best.R)) best=op;
			}
		}

		return best;
		
	}

	void deactivateOfferPath(Connection con, OfferPathRecord path) throws Exception
	{
		for (int n=0;n<path.offer.length;n++)
		{
			getMarketDB().deactivateOffer(con, path.offer[n].SN, path.offer[n].SN);
		}
	}

	void addToUserAcc(Connection con, byte[] UserFPR, byte[] IssuerFPR, long V) throws Exception
	{
		long UserAcc=getMarketDB().findUserAccRecord(con, UserFPR, IssuerFPR);
		getMarketDB().setUserAccRecord(con, UserFPR, IssuerFPR, UserAcc+V, (UserAcc==0));
	}
	
	void applyOffer(Connection con, byte[] UserFPR, OfferRecord offer, long VA, long VB) throws Exception
	{
		if ((VB*offer.RA/offer.RB)<VA)
			throw new DatabaseErrorException("Invalid offer path");
		
		offer.exchanged+=VA;
		
		getMarketDB().updateOfferExchanged(con, offer);
		
		if (!FPRUtils.compareFPR(UserFPR, offer.UserFPR)) {
			addToUserAcc(con, offer.UserFPR, offer.IssuerAFPR, -VA);
			addToUserAcc(con, offer.UserFPR, offer.IssuerBFPR, VB);
			
			if (UserFPR!=null) {
				addToUserAcc(con, UserFPR, offer.IssuerAFPR, VA);
				addToUserAcc(con, UserFPR, offer.IssuerBFPR, -VB);
			}
		}
		
	}
	
	void applyOfferPath(Connection con, byte[] UserFPR, OfferPathRecord path, long VA, long VB) throws Exception
	{
		int l=path.offer.length;
		
/*		if (UserFPR!=null)
		{
			long UserAccB=getMarketDB().findUserAccRecord(con, UserFPR, path.offer[0].IssuerBFPR);
			if (UserAccB==0) return;
			if (VB>UserAccB) VB=UserAccB;
		}*/

		if (VA!=-1) {
			VB=VA;
			
			for (int n=l-1;n>=0;n--) {
				VB=(VB*path.offer[n].RB+path.offer[n].RA-1)/path.offer[n].RA;
			}
		}
	
		for (int n=0;n<l;n++) {
			
			long VM=VB*path.offer[n].RA/path.offer[n].RB;
			if (VM>path.offer[n].avail)	VM=path.offer[n].avail;
			
			if (n==0) path.AB=VB;
			
			applyOffer(con, UserFPR, path.offer[n], VM, VB);
			
			VB=VM;
		}
		
		path.AA=VB;
		
	}

	public byte[] search(byte[] IssuerAFPR, byte[] IssuerBFPR, long RA, long RB, long VA, long VB) throws Exception
	{
		Connection con = null;
		
		OfferRecord offer = new OfferRecord();
		
		try
		{
			con = getConnection();
			getMarketDB().beginTransaction(con);
			getMarketDB().lockWrites(con,"Offer");
			getMarketDB().lockWrites(con,"UserAcc");
			
			String ResultStr="";

			while (true) {
				OfferPathRecord path = searchOfferPath(con, IssuerAFPR, IssuerBFPR, VA, VB);
				if (path==null) break;
				long CA=path.VA,CB=-1;
				if (VA!=-1) {
					if (VA<path.VA) CA=VA;
				}
				if (VB!=-1) {
					if (VB<path.VA) {CB=VB;CA=-1;}
				}
				if (path.VA==0) deactivateOfferPath(con, path);
					else applyOfferPath(con, null, path, CA, CB);
				if (VA!=-1) VA-=path.AA;
				if (VB!=-1) VB-=path.AB;
				
//				for (int i=0;i<path.offer.length;i++) ResultStr+=String.valueOf(path.offer[i].SN)+" ";
//				ResultStr+=String.valueOf((double)path.AA/ValueMultiplier)+" "+String.valueOf(path.AB/ValueMultiplier)+" "+String.valueOf(path.R)+"\n";
				
				ResultStr+= ValueDec.format((double)path.AA/(double)ValueMultiplier)+" "+
				  ValueDec.format((double)path.AB/(double)ValueMultiplier)+" "+
				  RatioDec.format((double)path.R)+"\n";
				
			}			
						
			byte[] cert = signMessage(ResultStr);
			return cert;
				
		}
		catch (SQLException ex)
		{
			con.rollback();
			throw new DatabaseErrorException(ex);
		}
		finally
		{
			con.rollback();
			closeConnection(con);
		}
		
	}
	
	
	public void addTestOffers(Connection con, byte[] UserFPR) throws Exception
	{
		OfferRecord offer=new OfferRecord();
		offer.active=true;
		offer.exchanged=0;
		offer.limit=1000000;
		offer.UserFPR=UserFPR;
		offer.IssuerAFPR=new byte[20];
		offer.IssuerBFPR=new byte[20];
		offer.nextSN=-1;
		offer.prevSN=-1;
		
		Random rnd=new Random();
		byte a,b;
		for (a=1;a<=10;a++) {
			offer.IssuerAFPR[0]=a;
			getMarketDB().setUserAccRecord(con,offer.UserFPR,offer.IssuerAFPR,((long)a-1)*100000,getMarketDB().findUserAccRecord(con,offer.UserFPR,offer.IssuerAFPR)==0);
			
			for (b=1;b<=10;b++) {
				offer.SN = getMarketDB().getOfferSN(con);
				offer.IssuerBFPR[0]=b;
				offer.RA=100+rnd.nextInt(100);
				offer.RB=100+rnd.nextInt(100);
				offer.logR=Math.round(Math.log((double)offer.RA/(double)offer.RB)*(double)1.0E16);
				getMarketDB().addOffer(con, offer);
			}
		}

		con.commit();

	}
	
	
	static long ValueMultiplier = 10000;
	static DecimalFormat ValueDec = new DecimalFormat("0.0000", new DecimalFormatSymbols(Locale.US));
	static DecimalFormat RatioDec = new DecimalFormat("0.00000", new DecimalFormatSymbols(Locale.US));


	public byte[] showOfferCert(Connection con, long SN) throws Exception
	{

		OfferRecord offer = getMarketDB().getFullOfferInfo(con, SN);

//		addTestOffers(con, offer.UserFPR);
		
		byte[] cert = signMessage(createOfferCert(offer));
		return cert;
	}
	
// if UserFPR==null, just deactivate offer SN	
	public byte[] offer(byte[] UserFPR, byte[] IssuerAFPR, byte[] IssuerBFPR, long RA, long RB, long VA, long VB, long SN, String reqText) throws Exception
	{
		Connection con = null;
		
		OfferRecord offer = new OfferRecord();
		
		try
		{
			con = getConnection();
			getMarketDB().beginTransaction(con);
			getMarketDB().lockWrites(con,"Offer");

			MessageDigest md = MessageDigest.getInstance("SHA");
			offer.reqHash = md.digest(reqText.getBytes("UTF-8"));
			
			long RSN=getMarketDB().findOfferHash(con, offer.reqHash);
			if (RSN!=-1) return showOfferCert(con, RSN);

			offer.SN = getMarketDB().getOfferSN(con);

			if (SN!=-1) getMarketDB().deactivateOffer(con, SN, offer.SN);

			offer.limit=-1;
			offer.logR=0;
			
			if (VA>0) VA*=ValueMultiplier;
			if (VB>0) VB*=ValueMultiplier;
			
			if (UserFPR!=null) {
				if (VA>0) offer.limit=VA;
				if (VB>0) offer.limit=VB*RA/RB;
				offer.logR=Math.round(Math.log((double)RA/(double)RB)*0x1000000);
			}
			
			offer.active = true;
			offer.UserFPR = UserFPR;
			offer.IssuerAFPR = IssuerAFPR;
			offer.IssuerBFPR = IssuerBFPR;
			offer.RA = RA;
			offer.RB = RB;
			offer.exchanged = 0;
			offer.nextSN = -1;
			offer.prevSN = SN;
			offer.reqText = reqText.getBytes("UTF-8");

			getMarketDB().addOffer(con, offer);
			
//			con.commit();
			
			while (true) {
				OfferPathRecord path = searchOfferPath(con, IssuerBFPR, IssuerAFPR, VB, VA);
				if (path==null) break;
				if (path.RA.multiply(BigInteger.valueOf(RA)).compareTo(path.RB.multiply(BigInteger.valueOf(RB)))==-1) break;
				
				OfferRecord o=getMarketDB().getSearchOfferInfo(con, offer.SN);
				if (o==null) break;
				path.addOffer(o,(VB!=-1));
				path.Calculate();
				if (VA<path.VA) path.VA=VA;
				if (VB<path.VB) path.VB=VB;
				applyOfferPath(con, UserFPR, path, path.VA, path.VB);
				if (VA!=-1) {
					VA-=path.AA;
					if (VA<=0) break;
				}
				if (VB!=-1) {
					VB-=path.AB;
					if (VB<=0) break;
				}
			}			
			
			con.commit();
			
			byte[] cert = signMessage(createOfferCert(getMarketDB().getFullOfferInfo(con, offer.SN)));
			return cert;
				
		}
		catch (SQLException ex)
		{
			con.rollback();
			throw new DatabaseErrorException(ex);
		}
		finally
		{
			closeConnection(con);
		}
		
	}
	
	
	
	static private SimpleDateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss z");
	
	public byte[] withdraw(byte[] UserFPR, byte[] IssuerFPR, long amount, String targetMD, String reqText) throws Exception
	{
		Connection con = null;
		try
		{
			con = getConnection();
			getMarketDB().beginTransaction(con);
			getMarketDB().lockWrites(con,"Draft");
			getMarketDB().lockWrites(con,"UserAcc");

			MessageDigest md = MessageDigest.getInstance("SHA");
			byte[] reqHash = md.digest(reqText.getBytes("UTF-8"));
			
			byte[] oldcert = getMarketDB().findDraftHash(con, reqHash);
			if (oldcert!=null) return oldcert;
			
			long DSN = getMarketDB().getDraftSN(con);
			
			amount*=ValueMultiplier;
			
			long _amount = getMarketDB().findUserAccRecord(con, UserFPR, IssuerFPR);
			
			if (_amount==0)
				throw new NotFoundException("Specified account is empty");

			if (amount==0) amount=_amount;
			_amount-=amount;
				
			if (_amount<0)
				throw new NotFoundException("Not enough ePoints on specified account");
			
			getMarketDB().setUserAccRecord(con, UserFPR, IssuerFPR, _amount, false);
				
//			logTransaction(newRec);
//			logTransaction(newObligation);
			
			//Date DateStart = new java.util.Date();
			
			GregorianCalendar DateStart = new GregorianCalendar();
			GregorianCalendar DateEnd = new GregorianCalendar();
			DateEnd.setTimeInMillis(DateStart.getTimeInMillis() + (365*24*3600*1000));
			
			byte[] cert = signMessage(createDraft(FPRUtils.parseFPR(getMarketFingerPrintText()), IssuerFPR, targetMD, amount,
					DSN, dateFormatter.format(DateStart.getTime()), dateFormatter.format(DateEnd.getTime()), reqText));

			getMarketDB().storeDraft(con, DSN, reqHash, cert);
			
			con.commit();
			
			return cert;
				
		}
		catch (SQLException ex)
		{
			con.rollback();
			throw new DatabaseErrorException(ex);
		}
		finally
		{
			closeConnection(con);
		}
		
	}
	
	public byte[] processDepositMessage(MessageDecryptionVerificationHandler signedMessage) throws RedcentProcessingException, SQLException, UnsupportedEncodingException
	{
		if (signedMessage != null)
		{
			String clearText = signedMessage.getClearText();
		
			if (signedMessage.isClearText().booleanValue())
			{
				Map data = getMessageHandler().parseMessage(clearText);
				
				if (data != null && data.size() != 0)
				{
//					signedMessage.getSigner().getFingerPrint();

					byte[] OwnerFPR = FPRUtils.parseFPR((String)data.get("ID"));
					
					if (!FPRUtils.compareFPR(OwnerFPR, FPRUtils.parseFPR(getMarketFingerPrintText())))
						throw new MalformedRequestException("Obligation not owned by this market");
					
					byte[] SignerFPR = FPRUtils.parseFPR(signedMessage.getSignersFingerprint());
					byte[] IssuerFPR = FPRUtils.parseFPR((String)data.get("ISSUER"));

					if (!FPRUtils.compareFPR(SignerFPR, IssuerFPR))
						throw new MalformedRequestException("Issuer key fingerprint does not match signature");
										
					byte[] UserFPR = FPRUtils.parseFPR((String)data.get("USER_NOTES"));
					String Value = (String)data.get("DIFFERENCE");
					
					if (UserFPR==null)
						throw new MalformedRequestException("No user key fingerprint in the User Notes of certificate");
					
					if (Value==null)
						throw new MalformedRequestException("No value specified in the certificate");
					
					long amount = Long.parseLong(Value);
					long OSN = Long.parseLong((String)data.get("SN")); 
					
					String message = createAccountInfo(UserFPR, depositAccount(UserFPR, IssuerFPR, OSN, amount));
					return signMessage(message);
				}
				else
					throw new MalformedRequestException("Obligation certificate could not be parsed");
			}
			else
				throw new BadsignatureException("Obligation certificates have to be clearsigned.");
			
		}
		else
			throw new MalformedRequestException("Illegal call to processDepositMessage with empty argument");
	}
	

	public String depositAccount(byte[] UserFPR, byte[] IssuerFPR, long OSN, long amount) throws RedcentProcessingException, SQLException
	{
		Connection con = null;
		try
		{
			con = getConnection();
			getMarketDB().beginTransaction(con);
			getMarketDB().lockWrites(con,"OCert");
			getMarketDB().lockWrites(con,"UserAcc");
			
			String Notes = FPRUtils.ToString(IssuerFPR)+" / "+getPaddedSerialNumber(OSN);

			amount*=ValueMultiplier;
			
			if (!getMarketDB().findOCert(con, IssuerFPR, OSN)) {
				
				long prevAmount = getMarketDB().findUserAccRecord(con, UserFPR, IssuerFPR);
				getMarketDB().setUserAccRecord(con, UserFPR, IssuerFPR, prevAmount+amount, (prevAmount==0));
				getMarketDB().addOCert(con, IssuerFPR, OSN);
				
				Notes = Notes + "     +" + String.valueOf(amount); 
				
			}
		    else Notes = Notes + "     *** already deposited";
			
			con.commit();

//			logTransaction(newRec);
//			logTransaction(newObligation);
			
			return Notes;
				
		}
		catch (SQLException ex)
		{
			con.rollback();
			throw new DatabaseErrorException(ex);
		}
		finally
		{
			closeConnection(con);
		}
	}
	
	public String createDraft(byte[] MarketFPR, byte[] IssuerFPR, String targetMD, long amount, long DSN, String DateStart, String DateEnd, String reqText) throws RedcentProcessingException, SQLException, ConflictException, NoSuchAlgorithmException
	{

		StringDataMap data = new StringDataMap();
		
		data.put("DRAWER", FPRUtils.ToString(MarketFPR));
		data.put("DRAWEE", FPRUtils.ToString(IssuerFPR));
		data.put("VALUE", String.valueOf(amount/ValueMultiplier));
		data.put("ID", targetMD);
		data.put("DATE", DateStart);
		data.put("MATURITY_DATE", DateStart);
		data.put("EXPIRY_DATE", DateEnd);
		data.put("MD_ALGO", "SHA1");
		data.put("NOTES", "DSN="+getPaddedSerialNumber(DSN)+"\n"+reqText);
		
		return getMessageHandler().createMessage(EpointCertificateTypes.DRAFT, data);
		
	}

	public String createOfferCert(OfferRecord offer) throws Exception //RedcentProcessingException, SQLException, ConflictException
	{

		StringDataMap data = new StringDataMap();
		
		data.put("SN", getPaddedSerialNumber(offer.SN));
		data.put("FPR", FPRUtils.ToString(offer.UserFPR));
		data.put("IA", FPRUtils.ToString(offer.IssuerAFPR));
		data.put("IB", FPRUtils.ToString(offer.IssuerBFPR));
		data.put("RA", offer.RA);
		data.put("RB", offer.RB);
		data.put("LIMIT", offer.limit/ValueMultiplier);
		data.put("EXCHANGED", ValueDec.format((double)offer.exchanged/(double)ValueMultiplier));
		
		String Notes="";
		if (offer.prevSN!=-1) Notes="prevSN="+getPaddedSerialNumber(offer.prevSN)+"\n";
		if (offer.nextSN!=-1) Notes=Notes+"nextSN="+getPaddedSerialNumber(offer.nextSN)+"\n";
		Notes=Notes+new String(offer.reqText, "UTF-8");
		
		data.put("MARKET_NOTES", Notes);
		
		return getMessageHandler().createMessage(MarketCertificateTypes.OFFER_CERTIFICATE, data);
		
	}
	
	public String createAccountInfo(byte[] UserFPR, String Notes) throws RedcentProcessingException, SQLException, ConflictException
	{
		Connection con = null;
		try
		{
			con = getConnection();
			getMarketDB().beginTransaction(con);
			getMarketDB().lockWrites(con,"UserAcc");
			
			StringDataMap data = new StringDataMap();
			
			data.put("USERFPR", FPRUtils.ToString(UserFPR));
			data.put("LIST", getMarketDB().getAccountInfo(con, UserFPR, ValueMultiplier));
			data.put("NOTES", Notes);
			
			return getMessageHandler().createMessage(MarketCertificateTypes.ACCOUNT_INFO, data);

		}
		catch (SQLException ex)
		{
			con.rollback();
			throw new DatabaseErrorException(ex);
		}
		finally
		{
			closeConnection(con);
		}
		
	}

	protected Connection getConnection() throws SQLException, DatabaseErrorException
	{
		Connection con = dataSource.getConnection();
		
		if (!con.isClosed())
			return con;
		else
			return dataSource.getConnection();
	}

	protected void closeConnection(Connection con) throws DatabaseErrorException
	{
		try
		{
			if (con != null && !con.isClosed())
				con.close();
		}
		catch (SQLException ex)
		{
			throw new DatabaseErrorException(ex);
		}
	}

	private String calcIdFromSecret(String rand) throws NoSuchAlgorithmException
	{
		if (rand == null)
			return null;
		
		// Calculate a certificate's ID from a given rand
		log.trace("Secret: " + rand);
		byte[] code = Base64.decodeBase64(rand.getBytes());
		MessageDigest md = MessageDigest.getInstance("SHA");
		String id = new String(Hex.encodeHex(md.digest(code))).toUpperCase();
		
		return id;
	}
	
	@Override
	public AuthorizedIssuer findAuthorizedIssuerByFingerprint(String authorizedIssuerFingerprint) throws NotFoundException, DatabaseErrorException
	{
		return null;
	}

	@Override
	public AuthorizedIssuer findAuthorizedIssuerByKeyID(long keyID) throws NotFoundException, DatabaseErrorException
	{
		return null;
	}
	
}
